package uoc.ei.ejemplos.modulo9.dns;


import uoc.ei.tads.Iterador;


/**
 * Interfaz DNS definida al enunciado de la prctica.
 * @author David Fguls
 * @author Jordi Alvarez (adaptacin para la versin 2.0.0 de la librera)
 *          Estructura de la Informacin,
 *          Universitat Oberta de Catalunya (UOC)
 * @version 2.0.0
 */
public interface DNS {
	
    /**
     * Mtodo que da de alta un dominio relacionndolo con una ip.
     * 
     * @param dominio objeto Dominio que contiene el dominio. ej: "www.uoc.edu".
     * @param ip  cadena que contiene la direccin ip del dominio. ej: "129.34.2.3".
     */
     public void altaDominio(Dominio d, IP ip);

     /**
      * Mtodo que da de baja un dominio.
      * 
      * @param dominio objeto Dominio que contiene el dominio. ej: "www.uoc.edu".
      */
     public void bajaDomini(Dominio d);

     /**
      * Mtodo que consulta la direccin ip de un dominio.
      * 
      * @param dominio objeto Dominio que contiene el dominio. ej: "www.uoc.edu".
      * @return ip  retorna una cadena que contiene la direccin ip del dominio. ej: "129.34.2.3".
      */
     public IP consultar(Dominio d);

     /**
      * Mtodo que consulta uno de los dominios ms visitados.
      * 
      * @param nm posicin dentro de del ranking (0..9). 
      * @return dominio retorna un objeto Dominio que contiene el nmero del dominio.
      */
     public Dominio masVisitados(int n);

     /**
      * Mtodo que inicializa un tld para hacer un recorrido.
      * 
      * @param t tld del cual queremos hacer un recorrido.
      */
     public Iterador<Entidad> entidades(TLD t);

}
 