package uoc.ei.ejemplos.modulo9.dns;

import uoc.ei.tads.*;

/**
 * Clase que encapsula la representacin escogida para guardar los hosts de cada una de las entidades 
 * de TLDs.<br>
 * En esta versin se ha usado la implementacin de los diccionarios mediante listas encadenadas redefiniendo el
 * mtodo consultar para poder reorganizar los hosts para optimizar las consultas de los ms recientemente 
 * consultados.
 * @author David Fguls
 * @author Jordi Alvarez (adaptacin para la versin 2.0.0 de la librera)
 *          Estructura de la Informacin,
 *          Universitat Oberta de Catalunya (UOC)
 * @version 2.0.0
 */
public class DiccionarioHosts extends DiccionarioListaImpl<Host,InformacionHost> {

	/**
	 * Mtodo que consulta los datos de un host a partir del suyo dominio. Est definiendo el mtodo 
	 * original de la DiccionarioListaImpl para permitir reordenar los hosts para optimizar
	 * las consultas de los ms recientemente consultados. <br>
	 * Para optimizar las consultas simplemente situamos al principio el host consultado.
	 * 
	 * @param clave el Dominio del host que queremos consultar.
	 * @return InformacionHost informacin del host.
	 */
	public InformacionHost consultar(Host clave) {
		InformacionHost element = null;
		// consultamos el elemento (clonamos implementacin de clase padre)
		RecorridoConAnterior<ClaveValor<Host,InformacionHost>> r=buscaPosicion(clave);
		Posicion<ClaveValor<Host,InformacionHost>> actual=r.actual();
		if (actual!=null) {
			element = actual.getElem().getValor();
			// borramos el elemento usando la posicin anterior.
			// lo hacemos siempre y cuando no se trate del primer elemento!!!
			Posicion<ClaveValor<Host,InformacionHost>> anterior=r.anterior();
			if (anterior!=null) {
				diccionario.borrar(anterior);
				// lo volvemos a aadir al principio de la lista
				diccionario.insertarAlPrincipio(actual.getElem());
			}
		}
		return element;
	}

} 