package uoc.ei.tads;
/**
 * Interfaz que define las operaciones bsicas de un conjunto.
 *
 * Los conjuntos son estructuras que almacenan elementos no repetidos. La
 * clase de los objetos debe disponer de una operacin de igualdad.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informacin,
 *          Universitat Oberta de Catalunya (UOC)
 * @version 2.0.0
 */
public interface Conjunto<E>  extends Contenedor<E>
{
   /**
    * Aade un elemento, si se puede.
    * @param elem  elemento que se quiere aadir al conjunto
    */
   public void insertar(E elem);

   /**
    * Comprueba si hay un elemento.
    * @param elem  elemento de referencia
    * @return  cierto o falso, segn si encuentra o no encuentra el elemento
    */
   public boolean esta(E elem);

   /**
    * Borra un elemento, si se puede.
    * @param elem  elemento de referencia
    * @return  elemento borrado; o null, si no era
    */
   public E borrar(E elem);
   
   /**
    * Aade los elementos de un segundo conjunto que no existen en el
    * conjunto actual, si se puede. Si encuentra un elemento equivalente, segn la
    * funcin de comparacin, lo sobrescribe.
    * @param conj  conjunto que se quiere unir al actual;
    *              puede ser vaco, pero no null
    * @exception ExcepcionParametroIncorrecto  si el conjunto es null
    */
   public void union(Conjunto<E> conj);

   /**
    * Borra del conjunto actual los elementos que no existen en un
    * segundo conjunto, si se puede.
    * @exception ExcepcionParametroIncorrecto  si el conjunto es null
    * @param conj  conjunto que se quiere interseccionar con el actual;
    *              puede ser vaco, pero no null
    */
   public void interseccion(Conjunto<E> conj);

   /**
    * Borra del conjunto actual los elementos que existen en un segundo
    * conjunto, si se puede.
    * @exception ExcepcionParametroIncorrecto  si el conjunto es null
    * @param conj  conjunto que se quiere sustraer del actual;
    *              puede ser vaco, pero no null
    */
   public void diferencia(Conjunto<E> conj);
}
