package uoc.ei.tads;

import java.io.Serializable;

/**
 * Interfaz que define las operaciones comunes de todos los TAD que se
 * caracterice por contener un conjunto de elementos. No se contemplan los
 * mtodos crea() ni destruye(): las clases que la implementen pueden
 * definirlos o dejar que el compilador use el constructor por
 * defecto, y el "garbage collector" para los objetos no referenciados.
 *
 * Tambin define, a efectos prcticos, el mtodo elementos() que permite
 * recorrer todos los elementos del contenedor mediante un iterador.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informacin,
 *          Universitat Oberta de Catalunya (UOC)
 * @version 2.0.0
 */
public interface Contenedor<E> extends Serializable
{
   /**
    * Comprueba si el contenedor est vaco.
    * @return  cierto si el contenedor est vaco
    */
   public boolean estaVacio();

   /**
    * Accesor de lectura del nmero de elementos que hay en el contenedor.
    * @return  nmero de elementos que hay en el contenedor
    */
   public int numElems();

   /**
    * Accesor de lectura de los elementos que hay en el contenedor.
    * Retorna una enumeracin. Se puede obtener un recorrido con un par de
    * lneas de cdigo: <PRE>
    * for ( Iterador it = tad.elementos(); it.haySiguiente(); )
    *    System.out.println(it.siguiente()); </PRE>
    * Enumerar es simplemente enunciar la una detrs la otra (las cosas
    * de una serie, las partes de un todo). Pero si el contenedor tiene definido
    * algn tipo de ordenacin o de recorrido, la enumeracin debe ser
    * consecuente y ofrecer los elementos por orden (FIFO, LIFO, inordre,
    * etc.), sin alterar el estado actual del contenedor.
    * @see  Iterador#haySiguiente()
    * @see  Iterador#siguiente()
    * @exception ExcepcionPosicionInvalida  si se quiere obtener el siguiente
    *            elemento de la enumeracin y ste no existe.
    * @return  enumeracin de los elementos del contenedor.
    */
   public Iterador<E> elementos();
}
