package uoc.ei.tads;
/**
 * Clase que implementa las operaciones de recorrido de las claves de los
 * elementos de un contenedor, por delegacin en un recorrido de posiciones.
 *
 * @see  ClaveValor
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informacin,
 *          Universitat Oberta de Catalunya (UOC)
 * @version 2.0.0
 */
class IteradorRecorridoClavesImpl<C,E>  extends IteradorRecorridoImpl<C,ClaveValor<C,E>>
{
   /** Atributo que en Java 1.5 determina la compatibilidad entre objetos
    * serialitzables de la misma clase. El identificador se calcula
    * mediante un mtodo de la clase Utilidades. */
   private static final long serialVersionUID = Utilidades.getSerialVersionUID();
	
	
   /**
    * Constructor con un parmetro.
    * @param recorrido  recorrido que proporciona los elementos
    */
   public IteradorRecorridoClavesImpl(Recorrido<ClaveValor<C,E>> recorrido)
   {
      super(recorrido);
   }

   /**
    * Accesor de lectura del elemento almacenado a la posicin.
    * @see  IteradorRecorridoImpl#siguiente()    
    * @param pos  posicin que contiene el elemento
    * @return  elemento contenido a la posicin
    */
   protected C getElem(Posicion<ClaveValor<C,E>> pos)
   {
      return  pos.getElem().getClave();
   }
}
