package uoc.ei.tads;

/**
 * Secuencia posicional que se caracteriza por disponer de operaciones
 * basadas en la posicin que ocupa un elemento dentro del contenedor.
 *
 * @see  Posicion
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informacin,
 *          Universitat Oberta de Catalunya (UOC)
 * @version 2.0.0
 */
public interface Lista<E>  extends Contenedor<E>
{
   /**
    * Aade un elemento al principio de la lista.
    * @param elem  elemento que se quiere aadir a la lista
    * @return  nueva posicin que contiene el elemento
    */
   public Posicion<E> insertarAlPrincipio(E elem);

   /**
    * Aade un elemento al final de la lista.
    * @param elem  elemento que se quiere aadir a la lista
    * @return  nueva posicin que contiene el elemento
    */
   public Posicion<E> insertarAlFinal(E elem);

   /**
    * Aade un elemento antes de la posicin recibida.
    * @param elem  elemento que se quiere aadir a la lista
    * @param pos  posicin de referencia
    * @exception ExcepcionPosicionInvalida  si la posicin es null o no
    *            vlida
    * @return  nueva posicin que contiene el elemento
    */
   public Posicion<E> insertarAntesDe(Posicion<E> pos, E elem);

   /**
    * Aade un elemento despus de la posicin recibida.
    * @param elem  elemento que se quiere aadir a la lista
    * @param pos  posicin de referencia
    * @exception ExcepcionPosicionInvalida  si la posicin es null o no
    *            vlida
    * @return  nueva posicin que contiene el elemento
    */
   public Posicion<E> insertarDespuesDe(Posicion<E> pos, E elem);

   /**
    * Borra la primera posicin de la lista.
    * @exception ExcepcionContenedorVacio  si la lista est vaca
    * @return  elemento que haba en la posicin
    */
   public E borrarPrimero();

   /**
    * Borra la posicin recibida.
    * @param pos  posicin que se quiere eliminar
    * @exception ExcepcionPosicionInvalida  si la posicin es null o no
    *            vlida
    * @return  elemento que haba en la posicin
    */
   public E borrar(Posicion<E> pos);

   /**
    * Borra la posicin siguiente.
    * @param pos  posicin anterior a la que se quiere eliminar
    * @exception ExcepcionPosicionInvalida  si la posicin es null o no
    *            vlida
    * @return  elemento que haba en la posicin siguiente
    */
   public E borrarSiguiente(Posicion<E> pos);

   /**
    * Reemplaza el elemento contenido a la posicin recibida.
    * @param elem  nuevo elemento
    * @param pos  posicin de referencia
    * @exception ExcepcionPosicionInvalida  si la posicin es null o no
    *            vlida
    * @return  elemento que haba a la posicin
    */
   public E reemplazar(Posicion<E> pos, E elem);

   /**
    * Intercambia los elementos contenidos a las posiciones recibidas.
    * @param pos1  primera de las dos posiciones de referencia
    * @param pos2  segunda de las dos posiciones de referencia
    * @exception ExcepcionPosicionInvalida  si la posicin es null o no
    *            vlida
    */
   public void intercambiar(Posicion<E> pos1, Posicion<E> pos2);
   

   /**
    * @return Un recorrido sobre las posiciones de la lista.
    */
   public Recorrido<E> posiciones();

}
