package uoc.ei.tads;
/**
 * Clase que representa una posicin en un contenedor implementado
 * mediante una representacin con vector. Tiene la utilidad de:
 * (a) A partir de un objeto de tipo Rango, la implementacin del
 * contenedor puede acceder en tiempo constante a la posicin del vector
 * que lo almacena; y
 * (b)poder cambiar elementos de ubicacin dentro del contenedor
 * sin invalidar referencias externas a elementos (de tipo Posicion/Rango)
 * que se hayan creado con anterioridad.
 *
 * @author  Jordi lvarez Canal
 * @author  Esteve Marin Gallis.
 *          Estructura de la Informacin,
 *          Universitat Oberta de Catalunya (UOC)
 * @version 2.0.0
 */
public class Rango<E>  implements Posicion<E> {

   /** Atributo que en Java 1.5 determina la compatibilidad entre objetos
    * serialitzables de la misma clase. El identificador se calcula
    * mediante un mtodo de la clase Utilidades. */
   private static final long serialVersionUID = Utilidades.getSerialVersionUID();
	
	
	/** Elemento contenido en el nodo. */
   private E elemento;

   /** ndice del elemento dentro del contenedor. */
   private int indice;

   /**
    * Constructor con dos parmetros.
    * @param idx  ndice de la posicin dentro de del contenedor.
    * @param elem  valor del elemento contenido a la posicin.
    */
   public Rango(int idx, E elem) {
      indice = idx;
      elemento = elem;
   }

   /**
    * Accesor de escritura del valor contenido en la posicin.
    * @param elem  nuevo valor del elemento contenido al nodo
    */
   public void setElem(E elem) { elemento = elem; }

   
   /**
    * Accesor de lectura del elemento contenido al nodo.
    * @return  elemento contenido al nodo
    */
   public E getElem() { return  elemento; }

   
   /**
    * Accesor de escritura del ndice de la posicin.
    * @param idx ndice de la posicin dentro de del contenedor.
    */
   public void setIndice(int idx) { indice = idx; }

   
   /**
    * Accesor de lectura de l'ndex de la posici.
    * @return  enter no negatiu
    */
   public int getIndice() { return  indice; }

   
   /**
    * Mtodo que define la conversin del objeto a String para
    * facilitar la depuracin del cdigo. Delega en el mtodo toString() de
    * el elemento almacenado en la posicin.
    * @return  cadena de caracteres representativa del elemento
    */
   public String toString()
   {
      return  (elemento == null) ? ("[" + indice + ",null]") :
              ("[" + indice + ","+ elemento.toString()+"]");
   }
}
