
package uoc.ei.tads;

import java.io.IOException;
import java.io.InputStream;

/** Esta clase proporciona un conjunto de mtodos estticos
 * que se utilizan en la implementacin de los TADs.
 * Tambin proporciona mtodos pblicos estticos con el objetivo
 * de que se puedan usar por los usuarios de la
 * librera de TADs.
 * 
 * @author Jordi Alvarez
 * @author Esteve Marin
 */
public class Utilidades {

	/** Versin actual de la librera de TADs. Se utiliza para
	 * dotar de un identificador a cada una de las clases de la
	 * librera. Este identificador se utiliza en la versin
	 * 1.5 de Java para determinar la compatibilidad entre objetos
	 * serializados.
	 */
	private static final String version = "2.00.00";
	

	/** Retorna la versin de la librera de TADs.
	 * @return La versin. El formato es x.yy.zz. Un cambio en la 'x'
	 * significa un cambio en el diseo de la librera como tal.
	 * Un cambio en 'yy' significa un cambio a nivel de funcionalidad;
	 * y un cambio a 'zz' significa un cambio donde se han solucionado
	 * problemas existentes a la versin anterior.
	 */
	public static String getVersion() {
		return version;
	}
	
	
	/** Retorna la versin de la librera de TADs como un long.
	 * @return el nmero correspondiente a la versin de la librera.
	 * El mtodo usado para la conversin es: versin 1.00.00 --> 10000.
	 */
	public static long getVersioLong() {
		String str=version.replaceAll("[.]","");
		return Long.parseLong(str);
	}
	
	/** Retorna un identificador que se puede usar por las clases
	 * de la librera para determinar la compatibilidad entre objetos
	 * serializados de la misma clase. Actualmente este identificador
	 * se corresponde con la versin de la librera.
	 * @return Identificador para la serializacin.
	 */
	public static long getSerialVersionUID() {
		return getVersioLong();
	}

	
	/** Lee una lnea del stream de entrada dado.
	 * @param str String de informacin que se saca por la salida estndar
	 * (til sobre todo si la entrada es la entrada estndar para sacar un
	 * mensaje del qu se pide. Si no, se puede dejar vaco.
	 * @param in Stream de entrada de donde se leer la lnea.
	 * @return Un String con el dato ledo.
	 * @throws IOException En caso de que se haya producido algn problema
	 * al leer el dato o sacar el mensaje por la salida estndar.
	 */
    public static String leerString(String str,InputStream in) throws IOException {
        if (str!=null) System.out.println(str);
        StringBuffer buffer=new StringBuffer();
        int c=in.read();
        while (c=='\n' || c=='\r') c=in.read();
        while (c!='\n' && c!='\r') {
            buffer.append((char)c);
            c=in.read();
        }
        return buffer.toString();
    }
    
    
	/** Retorna en un string una representacin textual del contenedor.
	 * @param nombreClase Nombre de la clase correspondiente al contenedor.
	 * @param rec Recorrido con todos los elementos del contenedor.
	 * @return un String con la representacin textual del contenedor.
	 */
	public static String toStringContenedor(String nombreClase,Recorrido rec) {
	      StringBuffer buffer = new StringBuffer("{"+nombreClase+": ");
	      while (rec.haySiguiente()) {
	      	Posicion kv=rec.siguiente();
	        buffer.append(kv.getElem());
	        if (rec.haySiguiente()) buffer.append(", ");
	      }
	      buffer.append("}");
	      return  buffer.toString();
		}


	/** Retorna en un string una representacin textual de un contenedor que delega
	 * su implementacin en otro contenedor (por ejemplo ConjuntoAVLImpl
	 * delega su implementacin en ArbolAVL).
	 * @param nombreClasse Nombre de la clase correspondiente al contenedor que
	 * usa delegacin.
	 * @param implementacion Contenedor Implementacin usado.
	 * @return un String con la representacin textual del contenedor.
	 */
	public static String toStringContenedorDelegacion(String nombreClasse,Contenedor implementacion) {
	   	return  "["+nombreClasse+": "+implementacion.toString()+"]";
		}

	
}
