package uoc.ei.tads.grafos;

import uoc.ei.tads.ExcepcionParametroIncorrecto;
import uoc.ei.tads.Utilidades;

/** 
 * Clase que representa una arista de un graf dirigido. La relacin es entre
 * dos vrtices del mismo dominio y va del vrtice origen al de destino.
 * @author Jordi Alvarez
 * @author Esteve Marin
 *          Estructura de la Informacin,
 *          Universitat Oberta de Catalunya (UOC)
 * @version 2.0.0 */
class AristaDImpl<TEtiqueta,TElem>  extends AristaImpl<TEtiqueta,TElem>
									implements AristaDirigida<TEtiqueta,TElem>
{

   private static final long serialVersionUID = Utilidades.getSerialVersionUID();
	
	
   /** Constructor con dos parmetros.
    * @param origen  vrtice origen de la arista
    * @param destino  vrtice destino de la arista
    * @exception ExcepcionParametroIncorrecto  si algn vrtice es null
    * @pre origen!=null && destino!=null, ExcepcionParametroIncorrecto
    */
   public AristaDImpl(VerticeDImpl<TElem,TEtiqueta> origen, VerticeDImpl<TElem,TEtiqueta> destino) {
      super(origen, destino);
   }

   /** Accesor de lectura del vrtice origen de la relacin dirigida.
    * @return  vrtice origen */
   public Vertice<TElem> getVerticeOrigen() { return  verticeA; }

   /** Accesor de lectura del vrtice destino de la relacin dirigida.
    * @return  vrtice destino */
   public Vertice<TElem> getVerticeDestino() { return  verticeB; }


}